/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.wsas.installer;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

import org.wso2.utils.AntBuildException;
import org.wso2.utils.AntBuildInvoker;
import org.wso2.utils.ArchiveManipulator;
import org.wso2.utils.InputReader;


/**
 * Manages the installation process of the WSO2 WSAS Embeddable distribution
 * on different application servers.
 */
public class InstallationManager {

    private static TomcatInstaller tomcatInstaller = new TomcatInstaller();
    private static JBossInstaller jbossInstaller = new JBossInstaller();
    private static GeronimoInstaller geronimoInstaller = new GeronimoInstaller();
    private static GenericInstaller genericInstaller = new GenericInstaller();

    // If we need to support any more app servers out of the box, we need to write an Installable
    // class, and add an instance of it to this array.
    private static final Installable[] INSTALLABLES
            = new Installable[]{InstallationManager.tomcatInstaller,
                                InstallationManager.jbossInstaller,
                                InstallationManager.geronimoInstaller,
                                InstallationManager.genericInstaller};

    // String array of installation modes
    // Future installation modes goes here and branch from selection - Ask for Installation Mode
    private static final String[] MODES = new String[]{"WSO2 WSAS Servlet Container Installation",
                                                       "WSO2 WSAS Eclipse WTP Plugins Installation"};

    public static void main(String[] args) {

        Properties props = new Properties();
        try {
            props.load(new FileInputStream("bin" + File.separator + "version.txt"));
        } catch (IOException e) {
            System.err.println("Cannot load version.txt file " + e);
            e.printStackTrace();
            System.exit(1);
        }

        String wsasVersion = props.getProperty("version");

        System.out.println("\n###########################################################");
        System.out.println("#                                                         #");
        System.out.println("#               WSO2 WSAS v" + wsasVersion + " Installation               #");
        System.out.println("#                                                         #");
        System.out.println("###########################################################\n");

        String javaHome = "";
        for (int i = 0; i < args.length; i++) {
            String arg = args[i];
            if (arg.equals("-tomcat_home")) {
                if (++i < args.length) {
                    InstallationManager.tomcatInstaller.setServerHome(args[i]);
                }
                continue;
            }

            if (arg.equals("-jboss_home")) {
                if (++i < args.length) {
                    InstallationManager.jbossInstaller.setServerHome(args[i]);
                }
                continue;
            }

            if (arg.equals("-geronimo_home")) {
                if (++i < args.length) {
                    InstallationManager.geronimoInstaller.setServerHome(args[i]);
                }
                continue;
            }

            if (arg.equals("-java_home")) {
                if (++i < args.length) {
                    javaHome = args[i];
                }
            }
        }

        System.out.println("Using Java Home : " + javaHome + "\n");

        // Ask for Installation Mode -      * 1) WSAS Application Server Installation
        //                                  * 2) WSAS Eclipse WTP Plugins Installation
        try {
            int installationMode = InstallationManager.getInstalltionMode();
            if (installationMode == 2) {    // 2) WSAS Eclipse WTP Plugins Installation
                EclipseWTPPluginInstaller.install(".", wsasVersion);
                System.exit(0);
            }
        } catch (InstallationException e) {
            System.err.println(" Installation failed : " + e);
            e.printStackTrace();
            System.exit(1);
        }

        // Extract the wso2wsas-installer.zip
        String installerZip = "wso2wsas-core-" + wsasVersion + "-installer" + ".zip";
        try {
            new ArchiveManipulator().extract("lib" + File.separator + installerZip,
                                             InstallerConstants.TMP_DIR);
        } catch (IOException e) {
            System.err.println("Cannot extract " + installerZip + " " + e);
            e.printStackTrace();
            System.exit(1);
        }

        // Ask for the WSO2 WSAS Home directory
        String wsasHome = InstallationManager.getWSASHome();
        System.setProperty("wso2wsas.home", wsasHome);
        AntBuildInvoker buildInvoker = new AntBuildInvoker(new File(InstallerConstants.INSTALLER_TMP_DIR +
                                                                    File.separator + "build.xml"));
        try {
            buildInvoker.invokeDefaultTarget(false);
        } catch (AntBuildException e) {
            e.printStackTrace();
            System.err.println("Installation failed " + e);
            System.exit(1);
        }

        try {
            // Ask to select an Application server
            Installable installable = InstallationManager.selectAppServer();
            installable.setWSASHome(wsasHome);
            installable.setJavaHome(javaHome);

            // App Server specific installation
            installable.install();

            // Cleanup files created during installation
            buildInvoker.invokeTarget("clean");
        } catch (Exception e) {
            System.err.println(" Installation failed : " + e);
            e.printStackTrace();
            System.exit(1);
        }
    }

    private static String getWSASHome() {
        File defaultWSASHome = new File(System.getProperty("user.home") +
                                        File.separator + ".wso2wsas");
        System.out.println("WSO2 WSAS Home directory is where the WSO2 WSAS database, conf files," +
                           " logs, and module & service repository will reside.");
        String wsasHome = defaultWSASHome.getAbsolutePath();
        do {
            System.out.print("Please enter the WSO2 WSAS Home directory [" +
                             defaultWSASHome.getAbsolutePath() + "]:");

            try {
                wsasHome = InputReader.readInput();
                if (wsasHome == null || wsasHome.trim().length() == 0) {
                    wsasHome = defaultWSASHome.getAbsolutePath();
                } else if (wsasHome.startsWith("~")) {
                    System.err.println("Please specify the path properly. The '~' character is not supported");
                    continue;
                } else if (new File(wsasHome).getCanonicalPath().
                        equals(new File(".").getCanonicalPath())) {
                    System.err.println("You cannot install WSO2 WSAS into this directory. " +
                                       "Please specify another directory.");
                    continue;
                }
            } catch (IOException e) {
                System.err.println(" Could not read WSO2 WSAS Home : " + e);
                System.exit(1);
            }
            File file = new File(wsasHome);
            if (file.isFile()) {
                System.err.println("WSO2 WSAS Home " + wsasHome + " is a file not a directory!");
            } else {
                break;
            }
        } while (true);
        return wsasHome.trim();
    }

    private static Installable selectAppServer() throws InstallationException {
        System.out.println("Please select your application server:");
        for (int i = 0; i < InstallationManager.INSTALLABLES.length; i++) {
            Installable installable = InstallationManager.INSTALLABLES[i];
            System.out.println(installable.getId() + ". " + installable.getDescription());
        }

        boolean foundValidServer = false;
        String serverId;
        Installable installable = null;
        while (!foundValidServer) {
            System.out.print(": ");
            try {
                serverId = InputReader.readInput();
            } catch (IOException e) {
                throw new InstallationException(e);
            }
            for (int i = 0; i < InstallationManager.INSTALLABLES.length; i++) {
                if (serverId.equals(InstallationManager.INSTALLABLES[i].getId())) {
                    foundValidServer = true;
                    installable = InstallationManager.INSTALLABLES[i];
                    break;
                }
            }
        }
        System.out.println();
        System.out.println("Installing on " + installable.getDescription() + "...");
        return installable;
    }


    private static int getInstalltionMode() throws InstallationException {
        System.out.println("\nPlease select your installation mode : [1/2]");
        for (int i = 0; i < InstallationManager.MODES.length; i++) {
            System.out.println(((i + 1) + ") ") + InstallationManager.MODES[i]);
        }

        boolean foundValidMode = false;
        int modeId = 0;
        while (!foundValidMode) {
            System.out.print(": ");
            try {
                modeId = Integer.parseInt(InputReader.readInput());
            } catch (IOException e) {
                throw new InstallationException(e);
            }
            if (modeId > 0 && modeId <= InstallationManager.MODES.length) {
                foundValidMode = true;
                break;
            } else {
                System.out.println("Invalid Selection !!");
            }
        }
        System.out.println();
        System.out.println("Selection: " + InstallationManager.MODES[modeId - 1]);
        return modeId;
    }
}
